#include <iostream>
#include <vector>
#include <string>
#include "svg.h"

using namespace std;

void svg_begin(double width, double height) {
    cout << "<?xml version='1.0' encoding='UTF-8'?>\n";
    cout << "<svg "
         << "width='" << width << "' "
         << "height='" << height << "' "
         << "viewBox='0 0 " << width << " " << height << "' "
         << "xmlns='http://www.w3.org/2000/svg'>\n";
}

void svg_end() {
    cout << "</svg>\n";
}

void svg_text(double left, double baseline, string text) {
    cout << "<text x='" << left << "' y='" << baseline << "'>" << text << "</text>\n";
}

void svg_rect(double x, double y, double width, double height, string stroke = "black", string fill = "black") {
    cout << "<rect x='" << x << "' y='" << y << "' width='" << width << "' height='" << height
         << "' stroke='" << stroke << "' fill='" << fill << "' />\n";
}

void draw_scale(double width, double height, double max_width, size_t step, double min, double max) {
    const double BLOCK_WIDTH = 10;
    const double SCALE_HEIGHT = 30;
    const double TEXT_OFFSET = 20;

    double scale_width = max_width;
    double scale_x = width - max_width;
    double scale_y = height - SCALE_HEIGHT;

    // Draw scale line
    svg_rect(scale_x, scale_y, scale_width, 2, "black", "black");

    // Draw scale marks and labels
    double step_size = scale_width / step;
    for (size_t i = 0; i <= step; i++) {
        double mark_x = scale_x + i * step_size;
        svg_rect(mark_x, scale_y, 2, 10, "black", "black");

        // Calculate value for label
        double value = min + (max - min) * i / step;
        string label = to_string(value);
        if (label.size() > 4) {
            label = label.substr(0, 4);
        }
        svg_text(mark_x - 10, scale_y + TEXT_OFFSET, label);
    }
}

void show_histogram_svg(const vector<size_t>& bins, size_t scale_step) {
    if (bins.empty()) {
        cerr << "Error: Empty bins vector\n";
        return;
    }

    const auto IMAGE_WIDTH = 400;
    const auto IMAGE_HEIGHT = 300;
    const auto TEXT_LEFT = 20;
    const auto TEXT_BASELINE = 20;
    const auto TEXT_WIDTH = 50;
    const auto BIN_HEIGHT = 30;
    const auto MAX_WIDTH = IMAGE_WIDTH - TEXT_WIDTH;

    svg_begin(IMAGE_WIDTH, IMAGE_HEIGHT);

    double top = 0;
    size_t max_count = bins[0];
    for (size_t bin : bins) {
        if (max_count < bin) {
            max_count = bin;
        }
    }

    for (size_t bin : bins) {
        const double bin_width = MAX_WIDTH * (static_cast<double>(bin) / max_count);
        svg_text(TEXT_LEFT, top + TEXT_BASELINE, to_string(bin));
        svg_rect(TEXT_WIDTH, top, bin_width, BIN_HEIGHT, "black", "red");
        top += BIN_HEIGHT;
    }

    draw_scale(IMAGE_WIDTH, IMAGE_HEIGHT, MAX_WIDTH, scale_step, 0, max_count);

    svg_end();
}
